package com.hammingweight.hammockexamples.example4;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;

import javax.microedition.io.HttpConnection;

import jmunit.framework.cldc10.TestCase;

import com.hammingweight.hammock.Hammock;
import com.hammingweight.hammock.mocks.io.MockDataInputStream;
import com.hammingweight.hammock.mocks.io.MockDataOutputStream;
import com.hammingweight.hammock.mocks.microedition.io.MockHttpConnection;
import com.hammingweight.hammockexamples.example4.HttpConnectionFactory;

public class HttpConnectionFactoryTest extends TestCase {

	// The OUT.
	private HttpConnectionFactory httpConnFactory;

	// The handler and mock(s).
	private Hammock hammock;
	private MockHttpConnection mockHttpConn;
	private MockDataOutputStream mockDataOutputStream;
	private MockDataInputStream mockDataInputStream;

	public void setUp() {
		this.hammock = new Hammock();
		this.mockHttpConn = new MockHttpConnection(this.hammock);
		this.mockDataOutputStream = new MockDataOutputStream(
				new ByteArrayOutputStream(), this.hammock);
		this.mockDataInputStream = new MockDataInputStream(
				new ByteArrayInputStream("".getBytes()), this.hammock);
		this.httpConnFactory = new HttpConnectionFactory("example.com");
	}

	public void testOpenAndClose() throws IOException {
		// Expectations.
		this.hammock.setExpectation(
				MockHttpConnection.MTHD_SET_REQUEST_METHOD_$_STRING,
				new Object[] { HttpConnection.GET });
		this.hammock.setExpectation(
				MockHttpConnection.MTHD_OPEN_DATA_OUTPUT_STREAM)
				.setReturnValue(this.mockDataOutputStream);
		this.hammock.setExpectation(
				MockHttpConnection.MTHD_OPEN_DATA_INPUT_STREAM).setReturnValue(
				this.mockDataInputStream);
		this.hammock.setExpectation(MockDataOutputStream.MTHD_CLOSE);
		this.hammock.setExpectation(MockDataInputStream.MTHD_CLOSE);
		this.hammock.setExpectation(MockHttpConnection.MTHD_CLOSE);

		// Exercise.
		this.httpConnFactory.openConnections(this.mockHttpConn);
		this.httpConnFactory.getInputConnection();
		this.httpConnFactory.getOutputConnection();
		this.httpConnFactory.closeConnections();
		// Close again, to check that the HttpConnection, DataOutputStream and
		// DataInputStream are closed only once.
		this.httpConnFactory.closeConnections();

		// Verify.
		this.hammock.verify();
	}

	public HttpConnectionFactoryTest() {
		super(1, "HttpConnectionFactoryTest");
	}

	public void test(int testNum) throws Throwable {
		switch (testNum) {
		case 0:
			testOpenAndClose();
			break;

		default:
			fail("No such test.");
		}
	}

}
