package com.hammingweight.hammockexamples.example5;

import jmunit.framework.cldc10.TestCase;

import com.hammingweight.hammock.ClassArgumentMatcher;
import com.hammingweight.hammock.Hamspy;
import com.hammingweight.hammock.MethodInvocation;
import com.hammingweight.hammockexamples.mocks.MockFlightBookingDAO;
import com.hammingweight.hammockexamples.mocks.MockLogger;

public class ViewBookingsListTest extends TestCase {

	// The spy handler
	private Hamspy hamspy;

	// The mock object(s)
	private MockLogger mockLogger;
	private MockFlightBookingDAO mockBookingDao;

	public void setUp() {
		this.hamspy = new Hamspy();
		this.mockLogger = new MockLogger(this.hamspy);
		this.mockBookingDao = new MockFlightBookingDAO(this.hamspy);
	}

	public void testConstructor() {
		// Arrange
		FlightBooking[] flights = {
				new FlightBooking("", "", "ABC123", "", ""),
				new FlightBooking("", "", "DEF456", "", ""),
				new FlightBooking("", "", "GHI789", "", "") };
		this.hamspy.setStubExpectation(
				MockFlightBookingDAO.MTHD_GET_ALL_BOOKINGS).setReturnValue(
				flights);

		// Act
		ViewBookingsList bookingsList = new ViewBookingsList(this.mockLogger,
				this.mockBookingDao);

		// Asserts.
		MethodInvocation[] mi = this.hamspy
				.getInvocations(MockLogger.MTHD_DEBUG_$_STRING);
		assertEquals(1, mi.length);
		assertEquals("ViewBookingsList constructed.", mi[0]
				.getMethodArguments()[0]);
		assertEquals("ABC123", bookingsList.getString(0));
		assertEquals("DEF456", bookingsList.getString(1));
		assertEquals("GHI789", bookingsList.getString(2));
	}

	public void testLogException() {
		// Arrange
		this.hamspy.setStubExpectation(
				MockFlightBookingDAO.MTHD_GET_ALL_BOOKINGS).setThrowable(
				new RuntimeException());

		// Act
		try {
			new ViewBookingsList(this.mockLogger, this.mockBookingDao);
			fail("Runtime Exception should have been thrown.");
		} catch (RuntimeException e) {
			// expected.
		}

		// Asserts.
		this.hamspy
				.getExpectation(
						MockLogger.MTHD_DEBUG_$_STRING_THROWABLE,
						new Object[] { "DAO threw exception.",
								new RuntimeException() }).setArgumentMatcher(1,
						new ClassArgumentMatcher()).setInvocationCount(1)
				.verify();
	}

	public ViewBookingsListTest() {
		super(2, "ViewBookingsListTest");
	}

	public void test(int testNum) throws Throwable {
		switch (testNum) {
		case 0:
			testConstructor();
			break;

		case 1:
			testLogException();
			break;

		default:
			fail("No such test.");
		}
	}

}
