/*
 * Maker.java
 *  
 * Copyright 2007 C.A. Meijer
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.hammingweight.hammockmaker;

import java.io.File;
import java.io.FileOutputStream;
import java.lang.reflect.Modifier;

/**
 * 
 * @author Carl
 */
public class Maker {
	private static void checkClassFinal(Class<?> clazz) throws Exception {
		int modifiers = clazz.getModifiers();
		if (Modifier.isFinal(modifiers)) {
			throw new Exception("The class " + clazz.getName() + " is final.");
		}
	}

	/** Creates a new instance of Builder */
	public Maker() {
	}

	public static void createMock(Class<?> clazzToMock, String mockPackage,
			String mockClazzName, boolean useCldc11, String destDir)
			throws Exception {
		checkClassFinal(clazzToMock);
		String packageDir = "";
		String pkgName = mockPackage;
		while (pkgName.indexOf('.') >= 0) {
			int index = pkgName.indexOf('.');
			packageDir += pkgName.substring(0, index);
			packageDir += File.separator;
			pkgName = pkgName.substring(index + 1);
		}
		packageDir += pkgName;
		if (!packageDir.equals("")) {
			packageDir += File.separator;
		}
		if ((destDir.charAt(destDir.length() - 1)) != File.separatorChar) {
			destDir += File.separator;
		}
		(new File(destDir + packageDir)).mkdirs();
		String fileName = destDir + packageDir + mockClazzName + ".java";
		ClassWrapper cw = new ClassWrapper(clazzToMock, mockPackage,
				mockClazzName, useCldc11);
		File outputFile = new File(fileName);
		FileOutputStream fos = new FileOutputStream(outputFile);
		fos.write(cw.toString().getBytes());
		fos.close();
		System.out.println("Mock object written to " + fileName);
	}

	public static void main(String[] args) throws Exception {
		if (((args.length != 3) && (args.length != 4))
				|| ((!args[0].equals("0")) && (!args[0].equals("1")))) {
			System.out
					.println("Usage: Maker [0|1] class_to_mock mocked_class [destination_dir]\n"
							+ "       where 0 or 1 denotes CLDC 1.0 or CLDC 1.1 and\n"
							+ "       where the class names must be fully qualified.");
			System.exit(0);
		}

		String clazzName = args[1];
		Class<?> clazz = Class.forName(clazzName);

		String mockClazz = args[2];
		int lastDotIndex = mockClazz.lastIndexOf('.');
		String mockPackage = "";
		if (lastDotIndex >= 0) {
			mockPackage = mockClazz.substring(0, lastDotIndex);
		}
		String mockClazzName = mockClazz.substring(
				mockClazz.lastIndexOf('.') + 1, mockClazz.length());
		String dest = args.length < 4 ? "." : args[3];
		Maker.createMock(clazz, mockPackage, mockClazzName,
				args[0].equals("1"), dest);
	}
}
