package com.hammingweight.hammock;

import org.easymock.classextension.EasyMock;

import junit.framework.TestCase;

public class InvocationMatcherTest extends TestCase {

	private static final IMockObject MOCK_OBJECT = EasyMock
			.createMock(IMockObject.class);

	private static final IMockObject MOCK_OBJECT2 = EasyMock
			.createMock(IMockObject.class);

	private static final MockMethod MOCK_METHOD = new MockMethod("FOOBAR", 3,
			null, true);

	public void testConstructorNullNotAllowed() {
		try {
			new InvocationMatcher(null);
			fail("Null can't be passed to the constructor.");
		} catch (NullPointerException e) {
			// expected.
		}
	}

	public void testTooManyExpectedArguments() {
		try {
			new InvocationMatcher(MOCK_METHOD, new Object[] { "foo",
					"bar", "baz", "4" });
			fail("Method takes 3 arguments.");
		} catch (HammockException e) {
			assertEquals(IHammockExceptionErrors.WRONG_NUMBER_OF_ARGS, e
					.getError());
		}
	}

	public void testTooFewExpectedArguments() {
		try {
			new InvocationMatcher(MOCK_METHOD, new Object[] { "foo",
					"bar" });
			fail("Method takes 3 arguments.");
		} catch (HammockException e) {
			assertEquals(IHammockExceptionErrors.WRONG_NUMBER_OF_ARGS, e
					.getError());
		}
	}

	public void testValidArguments() {
		MethodInvocation mi = new MethodInvocation(MOCK_METHOD, MOCK_OBJECT,
				new Object[] { "foo", "bar", "baz" });
		InvocationMatcher validator = new InvocationMatcher(
				MOCK_METHOD, new Object[] { "foo", "bar", "baz" });
		assertTrue(validator.isMatch(mi));
	}

	public void testBadArguments() {
		MethodInvocation mi = new MethodInvocation(MOCK_METHOD, MOCK_OBJECT,
				new Object[] { "foo", "bar", "Baz" });
		InvocationMatcher validator = new InvocationMatcher(
				MOCK_METHOD, new Object[] { "foo", "bar", "baz" });
		assertFalse(validator.isMatch(mi));
	}

	public void testWithArgumentMatcher() {
		MethodInvocation mi = new MethodInvocation(MOCK_METHOD, MOCK_OBJECT,
				new Object[] { "foo", "bar", "Baz" });
		InvocationMatcher validator = new InvocationMatcher(
				MOCK_METHOD, new Object[] { "foo", "bar", "baz" });
		validator.setArgumentMatcher(2, new IArgumentMatcher() {

			public boolean areArgumentsEqual(Object argumentExpected,
					Object argumentActual) {
				return argumentExpected.toString().toLowerCase().equals(
						argumentActual.toString().toLowerCase());
			}

		});
		validator.isMatch(mi);
	}

	public void testIgnoreArgument() {
		MethodInvocation mi = new MethodInvocation(MOCK_METHOD, MOCK_OBJECT,
				new Object[] { "foo", "bar", "Baz" });
		InvocationMatcher validator = new InvocationMatcher(
				MOCK_METHOD, new Object[] { "foo", "bar", null });
		validator.ignoreArgument(2);
		validator.isMatch(mi);
	}

	public void testMatchObject() {
		MethodInvocation mi = new MethodInvocation(MOCK_METHOD, MOCK_OBJECT,
				new Object[] { "foo", "bar", "baz" });
		MethodInvocation mi2 = new MethodInvocation(MOCK_METHOD, MOCK_OBJECT2,
				new Object[] { "foo", "bar", "baz" });
		InvocationMatcher validator = new InvocationMatcher(
				MOCK_METHOD, MOCK_OBJECT, new Object[] { "foo", "bar", "baz" });
		assertTrue(validator.isMatch(mi));
		assertFalse(validator.isMatch(mi2));
	}
}
